#include "events.h"
#include "io.h"

#include <stdlib.h>


void playerHit( Game *game, int attack ) {
    game->player.health -= attack;
    if ( game->player.health < 0 )
        game->player.health = 0;
}

int YesNoSelectWeapon( Game *game, char response ) {

    char weapon = readMapAtPlayer( game );

    switch ( response ) {
    case 'y':
    case 'Y':
        *mapAtPlayer( game ) = (char)game->player.weapon.type;
        game->player.weapon.type = getWeapon( weapon );
        game->player.weapon.attack = rand() % 10 + 5;
        break;
    case 'n':
    case 'N':
        break;
    default:
        addGameNotification( game, "Invalid response." );
        addGameNotification( game, "Select weapon. Found:" );
        addGameNotification( game, weaponType( getWeapon( weapon ) ) );
        addGameNotification( game, "Do you want to pick it up? (Y)es/(N)o" );
        return 1;
    }
    game->customResponseHandler = NULL;

    return 1;
}

void selectWeapon( Game *game, char weapon ) {
    addGameNotification( game, "Select weapon. Found:" );
    addGameNotification( game, weaponType( getWeapon( weapon ) ) );
    addGameNotification( game, "Do you want to pick it up? (Y)es/(N)o" );

    game->customResponseHandler = YesNoSelectWeapon;
}

void playerMove( Game *game, int newX, int newY ) {
    if ( readMapAt( game, newX, newY ) != '#' ) {
        game->player.position.x = newX;
        game->player.position.y = newY;
        char object = readMapAtPlayer( game );
        switch ( object ) {
        case 'a':
        case 'b':
        case 's':
            selectWeapon( game, object );
            break;
        case '+':
            ++game->player.health;
            *mapAtPlayer( game ) = ' ';
            break;
        case '@':
            ++game->player.weapon.attack;
            *mapAtPlayer( game ) = ' ';
            break;
        }
        return;
    }
    addGameNotification( game, "Player hit wall!" );
    playerHit( game, 1 );
}


int nextStep( Game *game ) {
    char c = readCommand();

    if ( game->customResponseHandler ) {
        return game->customResponseHandler( game, c );
    }

    switch ( c ) {
    case 'm':
        return 0;
    case 'a':
        playerMove( game, game->player.position.x - 1, game->player.position.y );
        break;
    case 's':
        playerMove( game, game->player.position.x, game->player.position.y + 1 );
        break;
    case 'w':
        playerMove( game, game->player.position.x, game->player.position.y - 1 );
        break;
    case 'd':
        playerMove( game, game->player.position.x + 1, game->player.position.y );
        break;
    default:
        addGameNotification( game, "Invalid command." );
        break;
    }

    if ( game->player.health )
        return 1;
    addGameNotification( game, "Game over!" );
    return 0;
}
