#include "game.h"
#include "io.h"
#include "fileOperations.h"

#include <stdlib.h>
#include <time.h>
#include <string.h>

char *copyString( const char *str ) {
    char *r = (char*)malloc( strlen( str ) + 1 );
    strcpy( r, str );
    return r;
}

const char *weaponType( enum Weapons type ) {
    switch ( type ) {
    case Axe:
        return "Axe";
    case Bow:
        return "Bow";
    case Sword:
        return "Sword";
    case NoWeapon:
        return "--nothing--";
    default:
        return "--unknown--";
    }
}

char *mapAtPlayer( Game *game ) {
    return mapAt( game, game->player.position.x, game->player.position.y );
}

char *mapAt( Game *game, int x, int y ) {
    return &game->map[ y * game->width + x ];
}
char readMapAtPlayer( const Game *game ) {
    return readMapAt( game, game->player.position.x, game->player.position.y );
}

char readMapAt( const Game *game, int x, int y ) {
    return game->map[ y * game->width + x ];
}

void clearGame( Game *game ) {
    game->height = 0;
    game->width = 0;
    if ( game->map )
        free( game->map );
    game->map = NULL;
    game->customResponseHandler = NULL;

    while ( game->notification ) {
        Notification *old = game->notification;
        game->notification = old->next;
        free( old );
    }
    if ( game->gameName )
        free( (void*)game->gameName );
    game->gameName = NULL;
    game->player.health = 0;
    game->player.killed = 0;
    game->player.position.x = 0;
    game->player.position.y = 0;
    game->player.weapon.attack = 0;
    game->player.weapon.type = NoWeapon;
}

void reloadGame( Game *game ) {
    if ( game->map ) {
        char *name = copyString( game->gameName );
        clearGame( game );
        loadGame( name, game );
        free( name );
        addGameNotification( game, "Map reloaded." );
    }
    else
        addGameNotification( game, "No map to reaload." );
}

void startGame( const char *filename ) {

    Game game;
    game.height = 0;
    game.width = 0;
    game.map = NULL;
    game.customResponseHandler = NULL;
    game.notification = NULL;
    game.gameName = NULL;
    game.player.health = 0;
    game.player.killed = 0;
    game.player.position.x = 0;
    game.player.position.y = 0;
    game.player.weapon.attack = 0;
    game.player.weapon.type = NoWeapon;

    srand( time( NULL ) );

    if ( filename )
        loadGame( filename, &game );


    do {
        switch ( mainMenu( &game ) ) {
        case EvLoadFile:
            clearGame( &game );
            loadGamePrompt( &game );
            break;
        case EvReloadFile:
            reloadGame( &game );
            break;
        case EvSaveFile:
            saveGamePrompt( &game );
            break;
        case EvPlay:
            if ( game.map ) {
                do {
                    printGame( &game );
                } while ( nextStep( &game ) );

            }
            else
                addGameNotification( &game, "No map loaded." );
            break;
        case EvHelp:
            help( &game );
            break;
        case EvQuit:
            return;
        }
    } while ( 1 );
}

enum Weapons getWeapon( char weapon ) {
    switch ( weapon ) {
    case '-':
        return NoWeapon;
    case 'a':
        return Axe;
    case 'b':
        return Bow;
    case 's':
        return Sword;
    default:
        return UnkownWeapon;
    }
}